/* create.c - Create empty databases.
   Copyright (C) 2004 Neal H. Walfield
   Written by Neal H. Walfield <neal@cs.uml.edu>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <error.h>
#include <argp.h>
#include <db.h>

#include "database.h"

static char doc[] = "db create -- database shell utilities";
static char args_doc[] = "[FILENAME...]";

static struct argp_option options [] = {
  { "force",		'f', 0,0, "Overwrite files." },
  { 0 }
};

static int force;

static error_t
parse_options (int key, char *arg, struct argp_state *state)
{
  switch (key)
    {
    case 'f':
      force = 1;
      break;

    default:
      return ARGP_ERR_UNKNOWN;

    case ARGP_KEY_NO_ARGS:
      {
	argp_usage (state);
	return EINVAL;
      }
    }
  return 0;
}

static struct argp argp = { options, parse_options, args_doc, doc };

int
dbsh_create (int argc, char *argv[])
{
  int err;
  int index;
  int failure = 0;

  err = argp_parse (&argp, argc, argv, 0, &index, 0);
  if (err)
    error (1, err, "argp_parse");

  if (index == argc)
    error (1, 0, "no file specified");

  for (; argv[index]; index ++)
    {
      DB *db;

      err = database_create (&db, argv[index], DB_BTREE, force);
      if (err)
	failure = 1;
      else
	database_close (db);
    }

  return failure ? 1 : 0;
}
