/* database.h - Database convenience functions.
   Copyright (C) 2004 Neal H. Walfield
   Written by Neal H. Walfield <neal@cs.uml.edu>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <error.h>
#include <db.h>
#include <assert.h>

static inline int
database_create (DB **dbp, const char *filename, DBTYPE type, int overwrite)
{
  int err;

  /* We only ever create stand alone databases.  */
  err = db_create (dbp, NULL, 0);
  if (err)
    {
      error (0, 0, "db_create: %s\n", db_strerror (err));
      return 1;
    }

  err = (*dbp)->open (*dbp, NULL, filename, NULL, type, 
		      DB_CREATE | (overwrite ? 0 : DB_EXCL), 0664);
  if (err)
    {
      (*dbp)->err (*dbp, err, "db->open (%s)", filename);
      (*dbp)->close (*dbp, 0);
      return 1;
    }

  return 0;
}

/* Open database in FILENAME.  Returns non-zero on failure.  */
static inline int
database_open (DB **dbp, const char *filename)
{
  int err;

  /* We only ever create stand alone databases.  */
  err = db_create (dbp, NULL, 0);
  if (err)
    {
      error (0, 0, "db_create: %s\n", db_strerror (err));
      return 1;
    }

  err = (*dbp)->open (*dbp, NULL, filename, NULL, DB_UNKNOWN, 0, 0);
  if (err)
    {
      (*dbp)->err (*dbp, err, "db->open (%s)", filename);
      (*dbp)->close (*dbp, 0);
      return 1;
    }

  return 0;
}

/* Close the database handle.  Will not fail.  */
static inline void
database_close (DB *db)
{
  error_t err;
  err = db->close (db, 0);
  assert (err == 0);
}
