/* db.c - Database front end.
   Copyright (C) 2004 Neal H. Walfield
   Written by Neal H. Walfield <neal@cs.uml.edu>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <error.h>
#include <errno.h>

const char *argp_program_version = PACKAGE_STRING;
const char *argp_program_bug_address = PACKAGE_BUGREPORT;

struct program
{
  char *name;
  int (*func) (int argc, char *argv[]);
};

extern int dbsh_create (int, char *[]);
extern int dbsh_get (int, char *[]);
extern int dbsh_put (int, char *[]);
extern int dbsh_search (int, char *[]);
extern int dbsh_del (int, char *[]);

static int help (int, char *[]);
static int usage (int, char *[]);
static int version (int, char *[]);

struct program progs[] =
  {
    { "create", dbsh_create },
    { "get", dbsh_get },
    { "put", dbsh_put },
    { "search", dbsh_search },
    { "del", dbsh_del },

    /* Normal arguments.  */
    { "--help", help },
    { "-?", help },
    { "--usage", usage },
    { "--version", version },
    { "-V", version },
  };

static int
help (int argc, char *argv[])
{
  int i;
  printf ("Usage: %s function [ARGUMENTS]\n"
	  "\n"
	  "  -?, --help\t\tGive this help list\n"
	  "      --usage\t\tGive a short usage message\n"
	  "  -V, --version\t\tPrint program version\n"
	  "\n",
	  program_invocation_short_name);

  printf ("Available functions:\n");
  for (i = 0; i < sizeof (progs) / sizeof (progs[0]); i ++)
    {
      if (progs[i].name[0] == '-')
	break;
      printf ("  %s\n", progs[i].name);
    }
  printf ("\n");

  printf ("Mandatory or optional arguments to long options are also "
		"mandatory or optional\n"
	  "for any corresponding short options.\n"
	  "\n"
	  "Report bugs to " PACKAGE_BUGREPORT "\n");
	  
  return argv ? 0 : 1;
}

static int
usage (int argc, char *argv[])
{
  printf ("Usage: %s function [ARGUMENTS]\n", program_invocation_short_name);
  return argv ? 0 : 1;
}

static int
version (int argc, char *argv[])
{
  printf ("%s\n", PACKAGE_STRING);
  return 0;
}

int
main (int argc, char *argv[])
{
  int i;

  if (argc < 2)
    return usage (0, 0);

  for (i = 0; i < sizeof (progs) / sizeof (progs[0]); i ++)
    {
      if (strcmp (progs[i].name, argv[1]) == 0)
	break;
    }

  if (i == sizeof (progs) / sizeof (progs[0]))
    error (1, 0, "unrecognized %s `%s'",
	   argv[1][0] == '-' ? "option" : "function", argv[1]);

  /* Modify the argument vector iff this is a real module.  */
  if (progs[i].name[0] != '-')
    {
      if (asprintf (&program_invocation_name, "%s %s", argv[0], argv[1]) < 0)
	error (1, errno, "");

      program_invocation_short_name
	= strrchr (program_invocation_name, '/') ?: program_invocation_name;

      argv[1] = program_invocation_name;
    }

  return progs[i].func (argc - 1, &argv[1]);
}
