%{
/* search-exp.y - Expression parser.
   Copyright (C) 2004 Neal H. Walfield
   Written by Neal H. Walfield <neal@cs.uml.edu>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <error.h>
#include <assert.h>

#include "search.h"
#include "search-exp.h"

extern int yylex ();

static int
yyerror (char *message)
{
  error (1, 0, "%s", message);
  return 0;
}
%}

%union {
  struct expression *expression;
  struct pattern *pattern;
}

%type <expression> expression
%token <pattern> PATTERN

%left 'o'
%left 'a'
%nonassoc '!' '(' ')'

%debug
%verbose

%%

program: expression	{ expressions = $1; }
	;

expression:
	PATTERN
		{ $$ = expression_make_pattern ($1); }
	| '!' expression
		{ $2->invert_logic = !$2->invert_logic; $$ = $2; }
	| expression 'a' expression
		{ $$ = expression_make_op (exp_and, $1, $3); }
	| expression expression
		{ $$ = expression_make_op (exp_and, $1, $2); }

	| expression 'o' expression
		{ $$ = expression_make_op (exp_or, $1, $3); }
	| '(' expression ')'
		{ $$ = $2; }
	;
