/* search.h - 
   Copyright (C) 2004 Neal H. Walfield
   Written by Neal H. Walfield <neal@cs.uml.edu>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef SEARCH_H
#define SEARCH_H

#include <regex.h>

#include "lib.h"

enum exp_type
  {
    exp_pattern,
    exp_or,
    exp_and,
  };

struct expression
{
  enum exp_type type;
  int invert_logic;

  union
  {
    struct
    {
      struct expression *a;
      struct expression *b;
    } expression;
    struct
    {
      struct pattern *pattern;
    } pattern;
  };
};

struct pattern
{
  regex_t re;
  char *pattern;
  int apply_to_key;
  int apply_to_value;
};

/* The expression tree.  Filled in by search-exp.y.  */
struct expression *expressions;

static inline struct expression *
expression_make_op (enum exp_type type,
		    struct expression *a, struct expression *b)
{
  struct expression *e = xcalloc (sizeof (struct expression), 1);
  e->type = type;
  e->expression.a = a;
  e->expression.b = b;
  return e;
}

static inline struct expression *
expression_make_pattern (struct pattern *p)
{
  struct expression *e = xcalloc (sizeof (struct expression), 1);
  e->type = exp_pattern;
  e->pattern.pattern = p;
  return e;
}

#endif /* SEARCH_H */
